import { Switch, Route, Router, useLocation } from "wouter";
import { queryClient } from "./lib/queryClient";
import { QueryClientProvider } from "@tanstack/react-query";
import { useEffect, useState, useRef } from "react";
import React from "react";
import NotFound from "@/pages/not-found";
import HomePage from "@/pages/home";
import IndividualReadingPage from "@/pages/individual-reading";
import CompatibilityReadingPage from "@/pages/compatibility-reading";
import FullCompatibilityReadingPage from "@/pages/full-compatibility-reading";
import ReadingSuccessPage from "@/pages/reading-success";
import ReadingCanceledPage from "@/pages/reading-canceled";
import AboutSoulContractsPage from "@/pages/about-soul-contracts";
import PrivacyPolicyPage from "@/pages/privacy-policy";
import TermsOfServicePage from "@/pages/terms-of-service";
import ReadingRedirectPage from "@/pages/reading-redirect";
import ArticlesPage from "@/pages/articles";
import ArticleDetailPage from "@/pages/article-detail";
import ArticleCategoryPage from "@/pages/article-category";
import AdminArticlesPage from "@/pages/admin/articles";
import AdminLoginPage from "@/pages/admin/login";
import AdminResetPasswordPage from "@/pages/admin/reset-password";
import AdminSettingsPage from "@/pages/admin/settings";
import TestLocationBasedPricing from "@/pages/compatibility-reading-test";
import { AdminAuthProvider } from "@/hooks/use-admin-auth";
import { ProtectedAdminRoute } from "@/components/protected-admin-route";
import { getReferralData, trackPageView } from "@/lib/utils";
import { Toaster } from "@/components/ui/toaster";
import MemoryMonitor from "@/components/performance-monitor";
import CustomErrorBoundary from "@/components/error-boundary";
import FetchErrorHandler from "@/components/fetch-error-handler";

// Memory management constants
const MEMORY_CHECK_INTERVAL = 30000; // Check memory every 30 seconds
const CRITICAL_MEMORY_THRESHOLD = 90; // 90% of available memory
const MEMORY_CLEANUP_INTERVAL = 60000; // Run cleanup every minute

/**
 * Memory Manager: Monitors and optimizes memory usage to prevent crashes
 * This helps prevent the blank purple screen by proactively managing memory
 */
function MemoryManager() {
  const [memoryUsage, setMemoryUsage] = useState<number | null>(null);
  const memoryWarningShown = useRef(false);
  
  // Memory monitoring and cleanup
  useEffect(() => {
    // Function to check memory usage and trigger cleanup if needed
    const checkMemory = () => {
      try {
        if ('performance' in window && 'memory' in (window.performance as any)) {
          const memory = (window.performance as any).memory;
          
          if (memory && memory.usedJSHeapSize && memory.jsHeapSizeLimit) {
            const usedPercentage = (memory.usedJSHeapSize / memory.jsHeapSizeLimit) * 100;
            setMemoryUsage(usedPercentage);
            
            // If memory usage is too high, trigger cleanup
            if (usedPercentage > CRITICAL_MEMORY_THRESHOLD && !memoryWarningShown.current) {
              console.warn(`[Memory Warning] High memory usage detected: ${usedPercentage.toFixed(2)}%`);
              memoryWarningShown.current = true;
              
              // Perform aggressive cleanup
              aggressiveMemoryCleanup();
            }
          }
        }
      } catch (error) {
        console.error("[Memory Monitor] Error checking memory:", error);
      }
    };
    
    // Regular memory check interval
    const memoryCheckInterval = setInterval(checkMemory, MEMORY_CHECK_INTERVAL);
    
    // Regular cleanup to prevent memory buildup
    const cleanupInterval = setInterval(() => {
      performMemoryCleanup();
    }, MEMORY_CLEANUP_INTERVAL);
    
    // Check memory immediately on mount
    checkMemory();
    
    return () => {
      clearInterval(memoryCheckInterval);
      clearInterval(cleanupInterval);
    };
  }, []);
  
  // Report critical memory issues to help with debugging
  useEffect(() => {
    if (memoryUsage && memoryUsage > CRITICAL_MEMORY_THRESHOLD) {
      trackPageView('memory_warning', {
        memory_usage: memoryUsage,
        url: window.location.href,
        timestamp: new Date().toISOString()
      });
    }
  }, [memoryUsage]);
  
  return null;
}

/**
 * Perform regular memory cleanup to prevent buildup
 */
function performMemoryCleanup() {
  try {
    // Clear any unused image cache
    if ('caches' in window) {
      caches.keys().then(cacheNames => {
        cacheNames.forEach(cacheName => {
          if (cacheName.includes('image-cache')) {
            caches.delete(cacheName);
          }
        });
      });
    }
    
    // Clear any expired local storage items
    if (localStorage) {
      try {
        // Look for items that might have expiration info
        const now = Date.now();
        Object.keys(localStorage).forEach(key => {
          try {
            if (key.includes('expires')) {
              const item = JSON.parse(localStorage.getItem(key) || '{}');
              if (item.expires && item.expires < now) {
                localStorage.removeItem(key);
              }
            }
          } catch (e) {
            // Skip items that can't be parsed
          }
        });
      } catch (e) {
        console.warn('[Memory Cleanup] Error cleaning localStorage:', e);
      }
    }
    
    // Clear React Query cache for old queries
    queryClient.invalidateQueries();
  } catch (error) {
    console.warn("[Memory Cleanup] Error during regular cleanup:", error);
  }
}

/**
 * Aggressive memory cleanup for when memory usage is critical
 * This helps recover from potential OOM situations
 */
function aggressiveMemoryCleanup() {
  try {
    console.warn("[Memory Cleanup] Performing aggressive memory cleanup");
    
    // Clear React Query cache completely
    queryClient.clear();
    
    // Clear all amplitude events
    if ((window as any).analytics && (window as any).analytics.events) {
      (window as any).analytics.events = [];
    }
    
    // Clear all unused caches
    if ('caches' in window) {
      caches.keys().then(cacheNames => {
        cacheNames.forEach(cacheName => {
          caches.delete(cacheName);
        });
      });
    }
    
    // Clear any heavyweight objects in memory
    if ((window as any).heavyweightObjects) {
      (window as any).heavyweightObjects = [];
    }
    
    // Force garbage collection if possible (this is a hint, not guaranteed)
    if ((window as any).gc) {
      (window as any).gc();
    }
    
    // Reset any large in-memory variables
    // These are application-specific and might not be applicable
    if ((window as any).chart_data_cache) {
      (window as any).chart_data_cache = {};
    }
    
    console.warn("[Memory Cleanup] Aggressive cleanup completed");
    
    // Report cleanup to analytics for monitoring
    trackPageView('memory_cleanup_performed', {
      url: window.location.href,
      timestamp: new Date().toISOString()
    });
  } catch (error) {
    console.error("[Memory Cleanup] Error during aggressive cleanup:", error);
  }
}

/**
 * Error recovery component to catch and handle runtime errors
 * that could cause the blank purple screen
 */
class ErrorBoundary extends React.Component<
  { children: React.ReactNode },
  { hasError: boolean; error: Error | null }
> {
  constructor(props: { children: React.ReactNode }) {
    super(props);
    this.state = { hasError: false, error: null };
  }

  static getDerivedStateFromError(error: Error) {
    return { hasError: true, error };
  }

  componentDidCatch(error: Error, errorInfo: React.ErrorInfo) {
    console.error("[ErrorBoundary] Caught an error:", error, errorInfo);
    
    // Track the error in analytics
    if (typeof window !== 'undefined' && (window as any).trackEvent) {
      (window as any).trackEvent('app_error', {
        error: error.message,
        stack: error.stack,
        url: window.location.href,
        timestamp: new Date().toISOString()
      });
    }
    
    // Try to recover by cleaning up memory
    aggressiveMemoryCleanup();
  }

  render() {
    if (this.state.hasError) {
      // Render fallback UI instead of blank purple screen
      return (
        <div className="error-recovery-container">
          <div className="error-recovery-content">
            <h2>Something went wrong</h2>
            <p>We're sorry, but we encountered an unexpected issue.</p>
            <button
              onClick={() => {
                // Clear the error state
                this.setState({ hasError: false, error: null });
                
                // Force a clean app state
                aggressiveMemoryCleanup();
                
                // Reload the page if needed
                if (confirm("Would you like to reload the page to recover?")) {
                  window.location.reload();
                }
              }}
              className="error-recovery-button"
            >
              Try to Recover
            </button>
          </div>
        </div>
      );
    }

    return this.props.children;
  }
}

// Tracks navigation and referral data
function NavigationTracker() {
  // Use the current location to track page views
  const [location] = useLocation();
  
  // Track page view with referral data when the location changes
  useEffect(() => {
    try {
      // Extract page name from the path
      const pageName = location === '/' 
        ? 'home' 
        : location.split('/')[1] || 'unknown';
      
      // Track the page view with current referral data
      const referralData = getReferralData();
      trackPageView(pageName, {
        full_path: location,
        ...referralData
      });
      
      // Also track in Amplitude if available
      if (typeof window !== 'undefined' && (window as any).analytics) {
        if ((window as any).analytics.identifyReferralSource) {
          (window as any).analytics.identifyReferralSource();
        }
      }
      
      // Clean up any previous page data to prevent memory leaks
      setTimeout(() => {
        performMemoryCleanup();
      }, 5000); // 5 seconds after navigation
    } catch (error) {
      console.error("[NavigationTracker] Error:", error);
    }
  }, [location]);
  
  return null; // This component doesn't render anything
}

/**
 * Component to detect and respond to low FPS/performance issues
 * that might indicate an impending blank purple screen
 */
function PerformanceMonitor() {
  const lastFrameTime = useRef(performance.now());
  const lowFpsCounter = useRef(0);
  const recoveryAttempted = useRef(false);
  
  useEffect(() => {
    const frameThreshold = 1000 / 15; // 15 fps minimum threshold
    let animationFrameId: number;
    
    const checkFrameRate = () => {
      const now = performance.now();
      const delta = now - lastFrameTime.current;
      
      // If frame took too long (low FPS), increment counter
      if (delta > frameThreshold) {
        lowFpsCounter.current++;
        
        // If we've had several consecutive slow frames, take action
        if (lowFpsCounter.current > 10 && !recoveryAttempted.current) {
          console.warn(`[Performance Warning] Low frame rate detected: ${(1000 / delta).toFixed(1)} FPS`);
          
          // Try to recover by cleaning memory
          performMemoryCleanup();
          recoveryAttempted.current = true;
          
          // Track performance issue
          trackPageView('performance_warning', {
            fps: (1000 / delta).toFixed(1),
            url: window.location.href
          });
        }
      } else {
        // Reset counters if we have good performance
        lowFpsCounter.current = 0;
        if (recoveryAttempted.current && lowFpsCounter.current === 0) {
          recoveryAttempted.current = false;
        }
      }
      
      lastFrameTime.current = now;
      animationFrameId = requestAnimationFrame(checkFrameRate);
    };
    
    animationFrameId = requestAnimationFrame(checkFrameRate);
    
    return () => {
      cancelAnimationFrame(animationFrameId);
    };
  }, []);
  
  return null;
}

function AppRouter() {
  return (
    <Switch>
      <Route path="/" component={HomePage} />
      <Route path="/reading/:name" component={IndividualReadingPage} />
      <Route path="/reading-redirect/:id" component={ReadingRedirectPage} />
      <Route path="/compatibility" component={CompatibilityReadingPage} />
      <Route path="/compatibility/:names" component={CompatibilityReadingPage} />
      <Route path="/full-compatibility/:names/:id" component={FullCompatibilityReadingPage} />
      <Route path="/reading-success" component={ReadingSuccessPage} />
      <Route path="/reading-canceled" component={ReadingCanceledPage} />
      <Route path="/about-soul-contracts" component={AboutSoulContractsPage} />
      <Route path="/privacy-policy" component={PrivacyPolicyPage} />
      <Route path="/terms-of-service" component={TermsOfServicePage} />
      <Route path="/compatibility-reading-test" component={TestLocationBasedPricing} />
      <Route path="/admin/login" component={AdminLoginPage} />
      <Route path="/admin/reset-password" component={AdminResetPasswordPage} />
      <ProtectedAdminRoute path="/admin/articles" component={AdminArticlesPage} />
      <ProtectedAdminRoute path="/admin/settings" component={AdminSettingsPage} />
      <Route path="/articles/category/:slug" component={ArticleCategoryPage} />
      <Route path="/articles/:slug" component={ArticleDetailPage} />
      <Route path="/articles" component={ArticlesPage} />
      <Route component={NotFound} />
    </Switch>
  );
}

function App() {
  // Cleanup when component unmounts to prevent memory leaks
  useEffect(() => {
    return () => {
      // Clear query cache when app unmounts
      queryClient.clear();
    };
  }, []);

  // Set up an interval to clear stale data every 5 minutes (reduced from 10)
  useEffect(() => {
    const intervalId = setInterval(() => {
      queryClient.invalidateQueries();
    }, 5 * 60 * 1000); // 5 minutes

    return () => clearInterval(intervalId);
  }, []);
  
  // Initialize referral tracking on app load
  useEffect(() => {
    try {
      // Track initial page view with referral data
      const referralData = getReferralData();
      
      // If we have UTM parameters or referrer data, track as a referred visit
      const hasReferralData = Object.keys(referralData).length > 0 && 
        (referralData.referrer_url || referralData.utm_source || referralData.ref);
      
      if (hasReferralData) {
        trackPageView('app_load_with_referral', referralData);
      } else {
        trackPageView('app_load_direct', { entry_page: window.location.pathname });
      }
    } catch (error) {
      console.error("[App] Error during initialization:", error);
    }
  }, []);

  return (
    <ErrorBoundary>
      <FetchErrorHandler>
        <QueryClientProvider client={queryClient}>
          <Router base="">
            <MemoryManager />
            <PerformanceMonitor />
            <NavigationTracker />
            <MemoryMonitor />
            <AdminAuthProvider>
              <AppRouter />
            </AdminAuthProvider>
            <Toaster />
          </Router>
        </QueryClientProvider>
      </FetchErrorHandler>
    </ErrorBoundary>
  );
}

export default App;
